using Microsoft.Extensions.Configuration;
using Phares.Shared;
using System.Text.Json;

namespace View_by_Distance.Date.Group.Models.Binder;

public class Configuration
{

    public bool? ByCreateDateShortcut { get; set; }
    public bool? ByDay { get; set; }
    public bool? ByHash { get; set; }
    public bool? BySeason { get; set; }
    public bool? ByWeek { get; set; }
    public bool? KeepFullPath { get; set; }
    public Property.Models.Configuration? PropertyConfiguration { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, Configuration? configuration)
    {
        if (configuration?.ByCreateDateShortcut is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static Models.Configuration Get(Configuration? configuration)
    {
        Models.Configuration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.PropertyConfiguration is null) throw new NullReferenceException(nameof(configuration.PropertyConfiguration));
        if (configuration.ByCreateDateShortcut is null) throw new NullReferenceException(nameof(configuration.ByCreateDateShortcut));
        if (configuration.ByDay is null) throw new NullReferenceException(nameof(configuration.ByDay));
        if (configuration.ByHash is null) throw new NullReferenceException(nameof(configuration.ByHash));
        if (configuration.BySeason is null) throw new NullReferenceException(nameof(configuration.BySeason));
        if (configuration.ByWeek is null) throw new NullReferenceException(nameof(configuration.ByWeek));
        if (configuration.KeepFullPath is null) throw new NullReferenceException(nameof(configuration.KeepFullPath));
        result = new(
            configuration.PropertyConfiguration,
            configuration.ByCreateDateShortcut.Value,
            configuration.ByDay.Value,
            configuration.ByHash.Value,
            configuration.BySeason.Value,
            configuration.ByWeek.Value,
            configuration.KeepFullPath.Value);
        return result;
    }

    public static Models.Configuration Get(IsEnvironment isEnvironment, IConfigurationRoot configurationRoot, Property.Models.Configuration propertyConfiguration)
    {
        Models.Configuration result;
        Configuration? configuration;
        if (isEnvironment is null)
#pragma warning disable IL3050, IL2026
            configuration = configurationRoot.Get<Configuration>();
#pragma warning restore IL3050, IL2026
        else
        {
            string environmentName = IsEnvironment.GetEnvironmentName(isEnvironment);
            string section = string.Concat(environmentName, ":", nameof(Configuration));
            IConfigurationSection configurationSection = configurationRoot.GetSection(section);
#pragma warning disable IL3050, IL2026
            configuration = configurationSection.Get<Configuration>();
#pragma warning restore IL3050, IL2026
        }
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}